@@ -1,4 +1,6 @@
-use super::super::common;
+use crate::common;
+use crate::common::create_testdata;
+use crate::common::test_data::get_test_hashes;
 
 #[tokio::test]
 async fn it_adds_files() {
@@ -20,23 +22,26 @@ async fn it_adds_binary_files() {
 #[tokio::test]
 async fn it_deletes_files() {
     let client = common::get_client();
-    client.delete_files(vec![]).await.unwrap();
+    client.delete_files(get_test_hashes()).await.unwrap();
 }
 
 #[tokio::test]
 async fn it_undeletes_files() {
     let client = common::get_client();
-    client.undelete_files(vec![]).await.unwrap();
+    create_testdata(&client).await;
+    client.undelete_files(get_test_hashes()).await.unwrap();
 }
 
 #[tokio::test]
 async fn it_archives_files() {
     let client = common::get_client();
-    client.archive_files(vec![]).await.unwrap();
+    create_testdata(&client).await;
+    client.archive_files(get_test_hashes()).await.unwrap();
 }
 
 #[tokio::test]
 async fn it_unarchives_files() {
     let client = common::get_client();
-    client.unarchive_files(vec![]).await.unwrap();
+    create_testdata(&client).await;
+    client.unarchive_files(get_test_hashes()).await.unwrap();
 }
